"""
AsyncPOW - https://github.com/totaldebug/asyncpow

Copyright (c) 2024 Steven Marks, Total Debug

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from pydantic import BaseModel

from asyncpow.models.common import (
    CreditModel,
    ExternalIdsModel,
    GenreModel,
    KeywordModel,
    ProductionCompanyModel,
    ProductionCountryModel,
    SpokenLanguagesModel,
    WatchProviderModel,
)


class EpisodeModel(BaseModel):
    """TV Episode model"""

    id: int
    name: str
    airDate: str | None
    episodeNumber: int
    overview: str
    productionCode: str
    seasonNumber: int
    showId: int
    stillPath: str | None
    voteAverage: int
    voteCount: int | None = None


class SeasonModel(BaseModel):
    """Model for TV Seasons"""

    id: int
    airDate: str | None = None
    episodeCount: int
    name: str
    overview: str
    posterPath: str
    seasonNumber: int


class CreatedByModel(BaseModel):
    """Model for Created by"""

    id: int
    name: str
    gender: int
    profilePath: str | None = None


class TvDetailsModel(BaseModel):
    """TV Details model"""

    id: int
    backdropPath: str
    posterPath: str
    contentRatings: dict
    createdBy: list[CreatedByModel]
    episodeRunTime: list[int]
    firstAirDate: str
    genres: list[GenreModel]
    homepage: str
    inProduction: bool
    languages: list[str]
    lastAirDate: str
    lastEpisodeToAir: EpisodeModel | None = None
    name: str
    nextEpisodeToAir: EpisodeModel | None = None
    networks: list[ProductionCompanyModel]
    numberOfEpisodes: int
    numberOfSeasons: int
    originCountry: list[str]
    originalLanguage: str
    originalName: str
    overview: str
    popularity: float
    productionCompanies: list[ProductionCompanyModel]
    productionCountries: list[ProductionCountryModel]
    spokenLanguages: list[SpokenLanguagesModel]
    seasons: list[SeasonModel]
    status: str
    tagline: str
    type: str
    voteAverage: float
    voteCount: int
    credits: CreditModel
    externalIds: ExternalIdsModel
    keywords: list[KeywordModel]
    watchProviders: list[WatchProviderModel]



"""
AsyncPOW - https://github.com/totaldebug/asyncpow

Copyright (c) 2024 Steven Marks, Total Debug

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from aiohttp import ClientSession
from yarl import URL

from asyncpow.models.tv import TvDetailsModel
from asyncpow.utils.http import request


class Tv:
    """
    Initialize the Tv object with the base URL and API key.
    """

    def __init__(self, base_url: URL, api_key: str, session: ClientSession) -> None:
        """
        Initialize the MovieAPI object with the base URL and API key.

        Args:
            base_url (str): The base URL for the media API.
            api_key (str): The API key for authentication.
            session (ClientSession): HTTP Session

        Returns:
            None
        """
        self.media_url = base_url.joinpath("tv")
        self.api_key = api_key
        self.session = session

    async def async_get_tv(
        self, tvId: int, lang: str = "en", raw_response: bool = False
    ) -> dict | TvDetailsModel:
        """
        Retrieves TV details by ID asynchronously.

        Args:
            tvId (int): The ID of the TV show.
            lang (str): The language for the response (default is "en").
            raw_response (bool): Flag to return raw response or TvDetailsModel (default is False).

        Returns:
            dict | TvDetailsModel: The raw response or TvDetailsModel object based on the raw_response flag.

        Examples:
            tv_details = await async_get_tv(12345, lang="en", raw_response=False)
        """
        params = {"language": lang}
        url = self.media_url.joinpath(str(tvId))

        headers = {"X-Api-Key": self.api_key}
        response = await request(self.session, url, params=params, headers=headers)
        return response if raw_response else TvDetailsModel(**response)



"""
AsyncPOW - https://github.com/totaldebug/asyncpow

Copyright (c) 2024 Steven Marks, Total Debug

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from typing import Literal

from pydantic import BaseModel

from asyncpow.models.common import (
    CreditModel,
    ExternalIdsModel,
    GenreModel,
    KeywordModel,
    ProductionCompanyModel,
    ProductionCountryModel,
    SpokenLanguagesModel,
    WatchProviderModel,
)


class RelatedVideoModel(BaseModel):
    """
    Data class representing a related video.
    """

    url: str
    key: str
    name: str
    size: int
    type: Literal[
        "Clip",
        "Teaser",
        "Trailer",
        "Featurette",
        "Opening Credits",
        "Behind the Scenes",
        "Bloopers",
    ]
    site: Literal["YouTube"]


class CollectionModel(BaseModel):
    """
    Data class representing a collection.
    """

    id: int
    name: str
    posterPath: str
    backdropPath: str


class RequestedByModel(BaseModel):
    """
    Data class representing the user who made the request.
    """

    id: int
    email: str
    username: str
    plexToken: str
    plexUsername: str
    userType: int
    permissions: int
    avatar: str
    createdAt: str
    updatedAt: str
    requestCount: int


class ModifiedByModel(BaseModel):
    """
    Data class representing the user who modified the item.
    """

    id: int
    email: str
    username: str
    plexToken: str
    plexUsername: str
    userType: int
    permissions: int
    avatar: str
    createdAt: str
    updatedAt: str
    requestCount: int


class RequestModel(BaseModel):
    """
    Data class representing a request.
    """

    id: int
    status: int
    media: str
    createdAt: str
    updatedAt: str
    requestedBy: RequestedByModel
    modifiedBy: ModifiedByModel
    is4k: bool
    serverId: int
    profileId: int
    rootFolder: str


class MediaInfoModel(BaseModel):
    """
    Data class representing media information.
    TODO: check this is used
    """

    id: int
    tmdbId: int
    tvdbId: int
    status: int
    requests: list[RequestModel]
    createdAt: str
    updatedAt: str


class MovieDetailsModel(BaseModel):
    """
    Data class representing a movie model.
    """

    id: int
    imdbId: str
    adult: bool
    backdropPath: str
    posterPath: str
    budget: int
    genres: list[GenreModel]
    homepage: str
    originalLanguage: str
    originalTitle: str
    overview: str
    popularity: float
    productionCompanies: list[ProductionCompanyModel]
    productionCountries: list[ProductionCountryModel]
    releaseDate: str
    releases: dict
    revenue: int
    runtime: int
    spokenLanguages: list[SpokenLanguagesModel]
    status: str
    tagline: str
    title: str
    video: bool
    voteAverage: float
    voteCount: int
    externalIds: ExternalIdsModel
    watchProviders: list[WatchProviderModel]
    keywords: list[KeywordModel]
    relatedVideos: list[RelatedVideoModel]
    credits: CreditModel
    collection: CollectionModel | None = None



"""
AsyncPOW - https://github.com/totaldebug/asyncpow

Copyright (c) 2024 Steven Marks, Total Debug

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from pydantic import BaseModel


class MovieResultModel(BaseModel):
    """
    Data class representing a movie search result.
    """

    id: int
    mediaType: str
    popularity: float
    posterPath: str
    backdropPath: str
    voteCount: int
    voteAverage: float
    genreIds: list[int]
    overview: str
    originalLanguage: str
    title: str
    originalTitle: str
    releaseDate: str
    adult: bool
    video: bool


class TvResultModel(BaseModel):
    """
    Data class representing a TV show search result.
    """

    id: int
    mediaType: str
    popularity: float
    posterPath: str
    backdropPath: str
    voteCount: int
    voteAverage: float
    genreIds: list[int]
    overview: str
    originalLanguage: str
    name: str
    originalName: str
    originCountry: list[str]
    firstAirDate: str


class PersonResultModel(BaseModel):
    """
    Data class representing a person search result.
    """

    id: int
    profilePath: str
    adult: bool
    mediaType: str
    knwonFor: list[MovieResultModel | TvResultModel]


class DiscoverTrendingModel(BaseModel):
    """
    Data class representing trending items in discovery.
    """

    page: int
    totalPages: int
    totalResults: int
    results: list[MovieResultModel | TvResultModel | PersonResultModel]


class WatchlistModel(BaseModel):
    """
    Data class representing an item in the watchlist.
    """

    ratingKey: str
    title: str
    mediaType: str
    tmdbId: int


class DiscoverWatchlistModel(BaseModel):
    """
    Data class representing watchlist items in discovery.
    """

    page: int
    totalPages: int
    totalResults: int
    results: list[WatchlistModel]



"""
AsyncPOW - https://github.com/totaldebug/asyncpow

Copyright (c) 2024 Steven Marks, Total Debug

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from aiohttp import ClientSession
from yarl import URL

from asyncpow.models.status import StatusAppDataModel, StatusModel
from asyncpow.utils.http import request


class Status:
    """
    Class to interact with status-related endpoints.

    Initialize the Status object with the base URL, API key, and session.
    """

    def __init__(self, base_url: URL, api_key: str, session: ClientSession) -> None:
        """
        Initialize the Status object with the base URL, API key, and session.

        Returns:
            None
        """

        self.base_url = base_url.joinpath("status")
        self.api_key = api_key
        self.session = session

    async def async_get_status(self, raw_response: bool = False) -> dict | StatusModel:
        """
        Summary:
            Asynchronously retrieves the status from the server.

        Args:
            raw_response (bool): Flag to determine whether to return the raw response (True) or an object (False). Default is False.

        Returns:
            dict | StatusModel: The status information as either a dictionary or a StatusModel object.
        """
        response = await request(self.session, self.base_url)
        return response if raw_response else StatusModel(**response)

    async def async_get_appdata(self, raw_response: bool = False) -> dict | StatusAppDataModel:
        """Retrieves the appdata from the server

        Args:
            raw_response (bool, optional): Flag to determine whether to return the raw response (True) or an object (False). Default is False.

        Returns:
            dict | StatusAppDataModel: The model object containing appdata items.
        """
        url = self.base_url.joinpath("appdata")
        response = await request(self.session, url)
        return response if raw_response else StatusAppDataModel(**response)



"""
AsyncPOW - https://github.com/totaldebug/asyncpow

Copyright (c) 2024 Steven Marks, Total Debug

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from typing import Literal

from pydantic import BaseModel

SortOptions = Literal["added", "modified", "mediaAdded"]


class UserModel(BaseModel):
    """
    Data class representing a user model.
    """

    id: int
    email: str
    username: str
    plexToken: str
    plexUsername: str
    userType: int
    permissions: int
    avatar: str
    createdAt: str
    updatedAt: str
    requestCount: str


class PageInfoModel(BaseModel):
    """
    Data class representing page information.
    """

    page: int
    pages: int
    results: int


class CastModel(BaseModel):
    """
    Data class representing a cast member.
    """

    id: int
    castId: int | None = None
    character: str
    creditId: str
    gender: int
    name: str
    order: int
    profilePath: str | None


class CrewModel(BaseModel):
    """
    Data class representing a crew member.
    """

    id: int
    creditId: str | None = None
    gender: int
    name: str
    job: str
    department: str
    profilePath: str | None


class CreditModel(BaseModel):
    """Data class representing Credits"""

    cast: list[CastModel]
    crew: list[CrewModel]


class GenreModel(BaseModel):
    """
    Data class representing a movie genre.
    """

    id: int
    name: str


class SpokenLanguagesModel(BaseModel):
    """
    Data class representing a spoken language.
    """

    englishName: str
    iso_639_1: str
    name: str


class ExternalIdsModel(BaseModel):
    """
    Data class representing external IDs.
    """

    facebookId: str | None = None
    freebaseId: str | None = None
    freebaseMid: str | None = None
    imdbId: str | None = None
    instagramId: str | None = None
    tvdbId: int | None = None
    tvrageId: int | None = None
    twitterId: str | None = None


class KeywordModel(BaseModel):
    """Data class representing a Keyword"""

    id: int
    name: str


class WatchProviderDetailsModel(BaseModel):
    """
    Data class representing a watch provider details.
    """

    displayPriority: int
    logoPath: str
    id: int
    name: str


class WatchProviderModel(BaseModel):
    """
    Data class representing a watch provider.
    """

    iso_3166_1: str
    link: str
    buy: list[WatchProviderDetailsModel]
    flatrate: list[dict]


class ProductionCompanyModel(BaseModel):
    """
    Data class representing a production company.
    """

    id: int
    logoPath: str | None
    originCountry: str
    name: str


class ProductionCountryModel(BaseModel):
    """
    Data class representing a production country.
    """

    iso_3166_1: str
    name: str



"""
AsyncPOW - https://github.com/totaldebug/asyncpow

Copyright (c) 2024 Steven Marks, Total Debug

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""

from typing import Optional

from aiohttp import ClientSession, hdrs
from yarl import URL

from asyncpow.models.common import SortOptions
from asyncpow.models.media import MediaFilterOptions, MediaModel, MediaModel2, MediaStatusOptions
from asyncpow.utils.http import request


class Media:
    """
    Class to interact with media-related endpoints.

    Initialize the Media object with the base URL, API key, and session.
    """

    def __init__(self, base_url: URL, api_key: str, session: ClientSession) -> None:
        """
        Initialize the MediaAPI object with the base URL and API key.

        Args:
            base_url (str): The base URL for the media API.
            api_key (str): The API key for authentication.
            session (ClientSession): HTTP Session

        Returns:
            None
        """
        self.media_url = base_url.joinpath("media")
        self.api_key = api_key
        self.session = session

    async def async_get_media(
        self,
        take: int = 20,
        skip: int = 0,
        filter: MediaFilterOptions | None = None,
        sort: SortOptions | None = None,
        raw_response: bool = False,
    ) -> dict | MediaModel:
        """
        Get media items based on specified parameters.

        Args:
            take (int): The number of items to retrieve (default is 20).
            skip (int): The number of items to skip (default is 0).
            filter (MediaFilterOptions): The filter option for media items (default is MediaFilterOptions.AVAILABLE).
            sort (SortOptions): The sorting option for media items (default is SortOptions.ADDED).
            raw_response (bool): Flag to determine whether to return the raw response (True) or an object (False). Default is False.

        Returns:
            dict | MediaModel: The media model object retrieved based on the parameters.
        """
        params: dict = {"take": take, "skip": skip}
        if filter:
            params["filter"] = filter
        if sort:
            params["sort"] = sort

        headers = {"X-Api-Key": self.api_key}
        response = await request(self.session, self.media_url, params=params, headers=headers)
        return response if raw_response else MediaModel(**response)

    async def async_post_media_status(
        self,
        mediaId: int,
        status: MediaStatusOptions,
        is4k: Optional[bool] = None,
        raw_response: bool = False,
    ) -> dict | MediaModel2:
        """
        Update the status of a media item with optional 4k flag.

        Args:
            mediaId (int): The ID of the media item.
            status (MediaStatusOptions): The status to set for the media item.
            is4k (Optional[bool]): Optional flag indicating 4k status.
            raw_response (bool): Flag to determine whether to return the raw response (True) or an object (False). Default is False.

        Returns:
            dict | MediaModel2: The model object representing the updated media item.
        """

        url = self.media_url.joinpath(str(mediaId), str(status))
        data = {"is4k": is4k} if is4k else {}
        headers = {"X-Api-Key": self.api_key}
        response = await request(
            self.session, url, method=hdrs.METH_POST, data=data, headers=headers
        )
        return response if raw_response else MediaModel2(**response)

    async def async_delete_media(self, mediaId: int) -> None:
        """
        Delete a media item by its ID.

        Args:
            mediaId (int): The ID of the media item to delete.

        Returns:
            None
        """

        url = self.media_url.joinpath(str(mediaId))
        headers = {"X-Api-Key": self.api_key}
        await request(self.session, url, method=hdrs.METH_DELETE, headers=headers)
